'''
************************************************************************
Copyright 2020 Institute of Theoretical and Applied Informatics, 
Polish Academy of Sciences https://www.iitis.pl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
************************************************************************

Data loading utility for the dataset
 `Comparison of surface thermal patterns of horses and donkeys in IRT images'
 IRT images of horses (H) and donkeys (D) with class annotation
 classes correspond to ROIs (1..15) which represent hand-annotated main muscle areas
 with 0 as background

@author: mromaszewski@iitis.pl
'''
import csv
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable

#a patch to your DS location
DS_DIR = '../' 
#H for horses, D for donkeys
ATYPES = ['H','D']
#normal animals
INDICES = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16]
#anomalies
ANOMALOUS_DONKEY_INDICES=[17,18]

def get_animal(name):
    """
    returns data and annotation for the animal
    parameters:
    name: animal name (use get_name())
    
    returns:
    data: 2D array of thermal data
    anno: 2D array with class map 
    """
    arr = np.load("{}data/da_{}.npz".format(DS_DIR,name))
    return arr['data'],arr['gt']


def get_name(atype='H',index=1):
    """
    returns animal name
    parameters:
    atype: type of animal from ['H','D']
    index: animal index: 1..16 (1..18 for donkeys)
    
    returns:
    animal name
    """
    assert atype in ATYPES
    return '{}.{}'.format(atype,index)

def plot_heatmap(data):
    """
    simple heatmap plot
    parameters:
    name: animal name (use get_name())
    """    
    ax=plt.gca()
    im = plt.imshow(data,cmap='nipy_spectral')
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="5%", pad=0.05)
    plt.colorbar(im, cax=cax)
    ax.set_axis_off()
    ax.get_xaxis().set_visible(False)
    ax.get_yaxis().set_visible(False)    

def print_features():
    """
    prints contents of the CSV file 
    """
    with open('{}/animal_features.csv'.format(DS_DIR)) as csvfile:
        reader = csv.reader(csvfile,delimiter=';')
        for row in reader:
            print (row)

def demo():
    """
    demo: data visualisation
    """
    for i in INDICES:
        for atype in ATYPES:
            aname = get_name(atype,i)
            data,anno = get_animal(aname)
            plt.subplot(1,3,1)
            plt.title("{}: data".format(aname))
            plot_heatmap(data)
            
            mt = np.mean(data[anno==1])
            print ("mean temperature for {} in ROI 1: {:0.2f}".format(aname,mt))
            
            plt.subplot(1,3,2)
            plt.title("{}: classes".format(aname))
            plot_heatmap(anno)
            plt.subplot(1,3,3)
            data[anno==0]=0
            plt.title("{}: ROIs".format(aname))
            plot_heatmap(data)

            plt.tight_layout(pad=0)
            plt.show()
            plt.close()    


if __name__ == "__main__":
    print_features()
    demo()